part of '../source_encoder.dart';

class DynamicElementEncoder extends ElementEncoder<DynamicElementImpl> {
  static const DynamicElementEncoder instance = DynamicElementEncoder._();

  const DynamicElementEncoder._() : super._(sdk.flutter_dynamic);

  @override
  void load(Map<String, Element> elements, DynamicElementImpl element) {
    final id = identifier(element);
    if (elements.containsKey(id)) return;
    elements[id] = element;
  }

  @override
  String identifier(DynamicElementImpl element) => target.identifier();

  @override
  String reference(DynamicElementImpl element) => element.name!;

  @override
  String referenceDeclaration(DynamicElementImpl element) => reference(element);

  @override
  String variable(DynamicElementImpl element) => identifier(element);

  @override
  String declaration(DynamicElementImpl element) =>
      DeclarationBuilder(kFlutterDynamic, identifier(element)).build(variable: variable(element));
}
