part of '../source_encoder.dart';

class EnumElementEncoder extends ElementEncoder<EnumElementImpl> {
  static const EnumElementEncoder instance = EnumElementEncoder._();

  const EnumElementEncoder._() : super._(sdk.flutter_enum);

  @override
  void load(Map<String, Element> elements, EnumElementImpl element) {
    final id = identifier(element);
    if (elements.containsKey(id)) return;
    elements[id] = element;
  }

  @override
  String identifier(EnumElementImpl element) => target.identifier(element.id);

  @override
  String reference(EnumElementImpl element) {
    final libraryAlias = LibraryEncoder.libraryAliasOf(element);
    return libraryAlias.isEmpty ? element.name : '$libraryAlias.${element.name}';
  }

  @override
  String referenceDeclaration(EnumElementImpl element) => reference(element);

  @override
  String variable(EnumElementImpl element) => identifier(element);

  @override
  String declaration(EnumElementImpl element) {
    final builder = DeclarationBuilder(kFlutterEnum, identifier(element));
    final accept = acceptElement(element);
    final enumReference = reference(element);
    if (accept) builder.parameterToken(enumReference);
    builder.parameter(name: 'library', value: LibraryEncoder.libraryVariableOf(element));
    final exports = LibraryEncoder.exportsFor(element);
    if (exports.isNotEmpty) builder.parameter(name: 'exports', value: '{${exports.join(', ')}}');
    builder.parameter(name: 'name', value: '\'${element.name}\'');
    if (accept) builder.parameter(name: 'values', value: '$enumReference.values');
    return builder.build(variable: variable(element));
  }
}
