part of '../source_encoder.dart';

class ParameterElementEncoder extends ElementEncoder<ParameterElement> {
  static const ParameterElementEncoder instance = ParameterElementEncoder._();
  static int _index = 1;

  const ParameterElementEncoder._() : super._(sdk.flutter_parameter);

  @override
  void load(Map<String, Element> elements, ParameterElement element) {
    final loadType = _typeLoader(elements);
    loadType(element.type);
  }

  @override
  String identifier(ParameterElement element) => target.identifier(element.id);

  @override
  String reference(ParameterElement element) {
    final name = element.name;
    return name.isEmpty ? 'p${_index++}' : name;
  }

  @override
  String referenceDeclaration(ParameterElement element) {
    final type = element.type;
    return '${TypeEncoder.of(type).reference(type)} ${reference(element)}';
  }

  @override
  String declaration(ParameterElement element) {
    final builder = DeclarationBuilder(kFlutterParameter, identifier(element));
    if (element.isNamed) builder.parameter(name: 'isNamed', value: 'true');
    if (element.isOptional) builder.parameter(name: 'isOptional', value: 'true');
    if (element.type.nullabilitySuffix == NullabilitySuffix.question)
      builder.parameter(name: 'isNullable', value: 'true');
    builder.parameter(name: 'name', value: '\'${reference(element)}\'');
    final type = element.type;
    builder.parameter(name: 'type', value: TypeEncoder.of(type).variable(type));
    final defaultValue = DefaultValueResolver(element).resolve();
    if (defaultValue != null) builder.parameter(name: 'defaultValue', value: defaultValue);
    return builder.build();
  }

  /// Used for unnamed parameter to provide declaration order index
  static void resetParameterIndex() => _index = 1;
}
