part of '../source_encoder.dart';

class TypeAliasElementEncoder extends ElementEncoder<TypeAliasElement> {
  static const TypeAliasElementEncoder instance = TypeAliasElementEncoder._();

  const TypeAliasElementEncoder._() : super._(sdk.flutter_type_alias);

  @override
  void load(Map<String, Element> elements, TypeAliasElement element) {
    final id = identifier(element);
    if (elements.containsKey(id)) return;
    final loadType = _typeLoader(elements);
    elements[id] = element;
    for (final typeParameter in element.typeParameters)
      TypeParameterElementEncoder.instance.load(elements, typeParameter);
    loadType(element.aliasedType);
  }

  @override
  String identifier(TypeAliasElement element) => target.identifier(element.id);

  @override
  String reference(TypeAliasElement element) {
    final aliasedElement = element.aliasedElement;
    if (aliasedElement is GenericFunctionTypeElement && aliasedElement.type.typeFormals.isNotEmpty) return kDynamic;
    final libraryAlias = LibraryEncoder.libraryAliasOf(element);
    return libraryAlias.isEmpty ? element.name : '$libraryAlias.${element.name}';
  }

  @override
  String referenceDeclaration(TypeAliasElement element) => reference(element);

  @override
  String variable(TypeAliasElement element) => identifier(element);

  @override
  String declaration(TypeAliasElement element) {
    final builder = DeclarationBuilder(kRawFlutterTypeAlias, identifier(element));
    final aliasedType = element.aliasedType;
    builder.parameter(name: 'library', value: LibraryEncoder.libraryVariableOf(element));
    final exports = LibraryEncoder.exportsFor(element);
    if (exports.isNotEmpty) builder.parameter(name: 'exports', value: '{${exports.join(', ')}}');
    builder.parameter(name: 'name', value: '\'${element.name}\'');
    final typeParameters = element.typeParameters;
    if (typeParameters.isNotEmpty)
      builder.parameter(
          name: 'typeParameters',
          value: '{${typeParameters.map(TypeParameterElementEncoder.instance.variable).join(', ')}}');
    builder.parameter(name: 'aliasedType', value: TypeEncoder.of(aliasedType).variable(aliasedType));
    return builder.build(variable: variable(element));
  }
}
