part of '../source_encoder.dart';

class TypeParameterElementEncoder extends ElementEncoder<TypeParameterElement> {
  static const TypeParameterElementEncoder instance = TypeParameterElementEncoder._();

  const TypeParameterElementEncoder._() : super._(sdk.flutter_type_parameter);

  @override
  void load(Map<String, Element> elements, TypeParameterElement element) {
    final id = identifier(element);
    if (elements.containsKey(id)) return;
    elements[id] = element;
  }

  @override
  String identifier(TypeParameterElement element) =>
      target.identifier(element.name, element.bound?.let((bound) => TypeEncoder.of(bound).identifier(bound)));

  @override
  String reference(TypeParameterElement element) => element.name;

  @override
  String referenceDeclaration(TypeParameterElement element) =>
      element.bound?.let((bound) => '${element.name} extends ${TypeEncoder.of(bound).reference(bound)}') ??
      element.name;

  @override
  String variable(TypeParameterElement element) => identifier(element);

  @override
  String declaration(TypeParameterElement element) {
    final builder = DeclarationBuilder(kFlutterTypeParameter, identifier(element));
    final bound = element.bound;
    builder.parameter(name: 'name', value: '\'${element.name}\'');
    if (bound != null) {
      final variable = TypeEncoder.of(bound).variable(bound);
      builder.parameterGetter(
          type: kFlutterType,
          name: 'bound',
          value: bound is InterfaceType && bound.typeArguments.isNotEmpty ? 'const $variable' : variable);
    }
    return builder.build(variable: variable(element));
  }
}
