// ignore_for_file: implementation_imports
import 'package:analyzer/dart/element/element.dart';
import 'package:analyzer/dart/element/nullability_suffix.dart';
import 'package:analyzer/dart/element/type.dart';
import 'package:analyzer/src/dart/element/element.dart';
import 'package:collection/collection.dart';
import 'package:flutterx_reflection_codegen/extension/code.dart';
import 'package:flutterx_reflection_codegen/src/encoder/target_sdk.dart';
import 'package:flutterx_reflection_codegen/src/encoder/top_level/encoder_library.dart';
import 'package:flutterx_reflection_codegen/src/encoder/utils/codegen_configuration.dart';
import 'package:flutterx_reflection_codegen/src/encoder/utils/declaration_builder.dart';
import 'package:flutterx_reflection_codegen/src/encoder/utils/default_value_resolver.dart';
import 'package:flutterx_reflection_codegen/src/encoder/wrapper/wrappers.dart';

part 'element/element_class.dart';
part 'element/element_dynamic.dart';
part 'element/element_enum.dart';
part 'element/element_function.dart';
part 'element/element_never.dart';
part 'element/element_parameter.dart';
part 'element/element_type_alias.dart';
part 'element/element_type_parameter.dart';
part 'element/element_void.dart';
part 'source_encoder_element.dart';
part 'source_encoder_type.dart';
part 'type/type_class.dart';
part 'type/type_dynamic.dart';
part 'type/type_enum.dart';
part 'type/type_function.dart';
part 'type/type_never.dart';
part 'type/type_type_alias.dart';
part 'type/type_type_parameter.dart';
part 'type/type_void.dart';

abstract class SourceEncoder<X> {
  final sdk target;

  const SourceEncoder._(this.target);

  void load(Map<String, Element> elements, X element) => throw UnsupportedError('not implemented $runtimeType');

  String identifier(X element) => throw UnsupportedError('not implemented $runtimeType');

  String reference(X element) => throw UnsupportedError('not implemented $runtimeType');

  String referenceDeclaration(X element) => throw UnsupportedError('not implemented $runtimeType');

  String variable(X element) => throw UnsupportedError('not implemented $runtimeType');

  String declaration(X element) => throw UnsupportedError('not implemented $runtimeType');
}

/// [tokens][[index]].use(<E extends xx>() => [innerCode]);
String encodeTokensUsage(String tokens, List<TypeParameterElement> typeParameters, String innerCode, [int index = 0]) {
  if (index == typeParameters.length) return innerCode;
  final parameter = typeParameters[index];
  final declaration = TypeParameterElementEncoder.instance.referenceDeclaration(parameter);
  return '$tokens[$index].use(<$declaration>() => ${encodeTokensUsage(tokens, typeParameters, innerCode, index + 1)})';
}

/// [List<E>, Map<List<E>, K>] -> [E, E, K]
Iterable<TypeParameterElement> findTypeParameters(Iterable<DartType> types,
    [Iterable<TypeParameterElement> exclude = const []]) sync* {
  for (final type in types) {
    if (type is TypeParameterType) {
      final bound = type.bound;
      if (!bound.isDynamic) yield* findTypeParameters([bound], exclude);
      final element = type.element;
      if (!exclude.contains(element)) yield element;
    } else if (type is ParameterizedType) {
      yield* findTypeParameters(type.typeArguments, exclude);
    }
  }
}

String referenceId(ClassElement classElement) =>
    [LibraryEncoder.libraryName(classElement), classElement.name].where((element) => element.isNotEmpty).join(':');

String parameterReferenceId(ParameterElement parameter) {
  final constructor = parameter.enclosingElement;
  if (constructor is! ConstructorElement) return parameter.name;
  return [referenceId(constructor.enclosingElement), constructor.name, parameter.name]
      .where((element) => element.isNotEmpty)
      .join(':');
}

bool acceptElement(Element element) =>
    element.isPublic && !element.hasDeprecated && !LibraryEncoder.isInternal(LibraryEncoder.libraryName(element));

bool acceptConstructor(ConstructorElement element) =>
    element.isPublic && !element.hasDeprecated && (!element.enclosingElement.isAbstract || element.isFactory);

bool acceptField(FieldElement element) => !element.isStatic && element.isPublic && !element.hasDeprecated;

bool acceptParameter(ParameterElement element) => !element.hasDeprecated || !element.isNamed || !element.isOptional;
