part of 'source_encoder.dart';

abstract class ElementEncoder<E extends Element> extends SourceEncoder<E> {
  static const Set<ElementEncoder> _encoders = {
    ClassElementEncoder.instance,
    DynamicElementEncoder.instance,
    EnumElementEncoder.instance,
    FunctionElementEncoder.instance,
    NeverElementEncoder.instance,
    ParameterElementEncoder.instance,
    TypeAliasElementEncoder.instance,
    TypeParameterElementEncoder.instance,
    VoidElementEncoder.instance,
  };

  const ElementEncoder._(sdk target) : super._(target);

  factory ElementEncoder(sdk target) =>
      _encoders.firstWhere((encoder) => encoder.target == target) as ElementEncoder<E>;

  factory ElementEncoder.of(E element) => ElementEncoder(_targetOfElement(element));

  @override
  void load(Map<String, Element> elements, E element);

  @override
  String identifier(E element);

  @override
  String reference(E element);

  @override
  String referenceDeclaration(E element);

  @override
  String variable(E element);

  @override
  String declaration(E element);
}

sdk _targetOfElement(Element element) {
  if (element is ClassElement) return element.isEnum ? sdk.flutter_enum : sdk.flutter_class;
  if (element is DynamicElementImpl) return sdk.flutter_dynamic;
  if (element is TypeParameterElement) return sdk.flutter_type_parameter;
  if (element is FunctionTypedElement) return sdk.flutter_function;
  if (element is TypeAliasElement) return sdk.flutter_type_alias;
  if (element is ParameterElement) return sdk.flutter_parameter;
  if (element is VoidElementWrapper) return sdk.flutter_void;
  if (element is NeverElementImpl) return sdk.flutter_never;
  throw StateError('unsupported ${element.runtimeType}(${element.displayName})');
}
