part of 'source_encoder.dart';

abstract class TypeEncoder<T extends DartType> extends SourceEncoder<T> {
  static const Set<TypeEncoder> _encoders = {
    ClassTypeEncoder.instance,
    DynamicTypeEncoder.instance,
    EnumTypeEncoder.instance,
    FunctionTypeEncoder.instance,
    NeverTypeEncoder.instance,
    TypeAliasTypeEncoder.instance,
    TypeParameterTypeEncoder.instance,
    VoidTypeEncoder.instance,
  };

  const TypeEncoder._(sdk target) : super._(target);

  factory TypeEncoder(sdk target) => _encoders.firstWhere((encoder) => encoder.target == target) as TypeEncoder<T>;

  factory TypeEncoder.of(T type) => TypeEncoder(_targetOfType(type));

  ElementEncoder get elementEncoder;

  @override
  void load(Map<String, Element> elements, T type); // ignore: avoid_renaming_method_parameters

  @override
  String identifier(T type); // ignore: avoid_renaming_method_parameters

  @override
  String reference(T type); // ignore: avoid_renaming_method_parameters

  @override
  String referenceDeclaration(T type); // ignore: avoid_renaming_method_parameters

  @override
  String variable(T type); // ignore: avoid_renaming_method_parameters

  @override
  String declaration(T type); // ignore: avoid_renaming_method_parameters
}

sdk _targetOfType(DartType type) {
  if (type.aliasElement != null) return sdk.flutter_type_alias;
  if (type is InterfaceType) return type.element.isEnum ? sdk.flutter_enum : sdk.flutter_class;
  if (type is DynamicType) return sdk.flutter_dynamic;
  if (type is TypeParameterType) return sdk.flutter_type_parameter;
  if (type is FunctionType) return sdk.flutter_function;
  if (type is VoidType) return sdk.flutter_void;
  if (type is NeverType) return sdk.flutter_never;
  throw StateError('unsupported ${type.runtimeType}(${type.getDisplayString(withNullability: true)})');
}

void Function(DartType) _typeLoader(Map<String, Element> elements) {
  void _loadType(DartType type) => TypeEncoder.of(type).load(elements, type);
  return _loadType;
}

String nullabilityIdentifier(DartType type) => type.nullabilitySuffix == NullabilitySuffix.question ? 'q' : '';

String nullabilitySuffix(DartType type) => type.nullabilitySuffix == NullabilitySuffix.question ? '?' : '';
