import 'package:analyzer/dart/element/element.dart';
import 'package:flutterx_reflection_codegen/extension/string.dart';
import 'package:flutterx_reflection_codegen/src/encoder/source_encoder.dart';
import 'package:flutterx_reflection_codegen/src/encoder/target_sdk.dart';
import 'package:flutterx_reflection_codegen/src/encoder/top_level/encoder_library.dart';
import 'package:flutterx_reflection_codegen/src/encoder/utils/codegen_configuration.dart';
import 'package:flutterx_reflection_codegen/src/writer_framework.dart';

class FrameworkEncoder {
  static const FrameworkEncoder instance = FrameworkEncoder._();

  const FrameworkEncoder._();

  String encode(FrameworkWriter subject) {
    final entries = subject.entries;
    final current = subject.current;
    CodegenConfig.load(subject);
    final imports = <String>['import \'package:flutterx_reflection/flutterx_reflection.dart\';'];
    final libraryVariables = <String>[];
    final declaredTypes = <String>{};
    final body = StringBuffer();
    final className = _encodeFrameworkName(current);
    for (final type in entries) {
      final encoder = ElementEncoder.of(type);
      if (type is TypeDefiningElement && type is! TypeParameterElement) declaredTypes.add(encoder.variable(type));
      body.writeln(encoder.declaration(type));
    }
    for (final libraryName in LibraryEncoder.usedAliases) {
      if (!LibraryEncoder.isInternal(libraryName)) imports.add(LibraryEncoder.encodeImport(libraryName));
    }
    imports.sort(sortImports);
    for (final libraryName in LibraryEncoder.usedVariables) {
      libraryVariables.add(LibraryEncoder.encodeVariable(libraryName));
    }
    libraryVariables.sort((a, b) => sortImports(a.split('=').last, b.split('=').last));
    return '''
    // ignore_for_file: always_declare_return_types
    // ignore_for_file: avoid_types_on_closure_parameters
    // ignore_for_file: avoid_private_typedef_functions
    // ignore_for_file: deprecated_member_use
    // ignore_for_file: do_not_use_environment
    // ignore_for_file: implementation_imports
    // ignore_for_file: invalid_use_of_visible_for_testing_member
    // ignore_for_file: library_private_types_in_public_api
    // ignore_for_file: prefer_collection_literals
    // ignore_for_file: prefer_single_quotes
    // ignore_for_file: prefer_void_to_null
    // ignore_for_file: unused_element

    ${imports.join('\n')}

    typedef $kFlutterParameter = FlutterParameter;
    typedef $kDefaultValueResolver = DefaultValueResolver;
    typedef $kFlutterType = FlutterType;
    typedef $kDefinedType = DefinedType;
    typedef $kFlutterTypeAlias = FlutterTypeAlias;
    typedef $kRawFlutterTypeAlias = RawFlutterTypeAlias;
    typedef $kParametrizedFlutterTypeAlias = ParametrizedFlutterTypeAlias;
    typedef $kFlutterClass = FlutterClass;
    typedef $kRawFlutterClass = RawFlutterClass;
    typedef $kParametrizedFlutterClass = ParametrizedFlutterClass;
    typedef $kFlutterConstructor = FlutterConstructor;
    typedef $kFlutterField = FlutterField;
    typedef $kFactoryContext = FactoryContext;
    typedef $kParameterReader = ParameterReader;
    typedef $kFlutterDynamic = FlutterDynamic;
    typedef $kFlutterEnum = FlutterEnum;
    typedef $kFlutterFunction = FlutterFunction;
    typedef $kRawFlutterFunction = RawFlutterFunction;
    typedef $kParametrizedFlutterFunction = ParametrizedFlutterFunction;
    typedef $kFunctionContext = FunctionContext;
    typedef $kFlutterNever = FlutterNever;
    typedef $kFlutterTypeParameter = FlutterTypeParameter;
    typedef $kTypeToken<T> = TypeToken<T>;
    typedef $kFlutterVoid = FlutterVoid;

    class $className {
      static const $kSet<$kDefinedType> allTypes = {${declaredTypes.join(', ')}};
      static final $kMap<$kString, $kDefinedType> allTypesMap = $kMap.fromEntries(allTypes.map((type) => MapEntry(type.referenceId, type)));

      const $className._();

      static $kDefinedType lookUp($kString referenceId) {
        final type = allTypesMap[referenceId];
        if (type == null) return allTypes.firstWhere((element) => _Dt.allReferenceIds(element).contains(referenceId), orElse: () => throw StateError('class not found: \$referenceId'));
        return type;
      }
    }

    ${libraryVariables.join('\n')}
    $body'''
        .trimMargin();
  }

  static String _encodeFrameworkName(LibraryElement sdkLibrary) {
    final libraryName = LibraryEncoder.libraryName(sdkLibrary);
    var fileName = libraryName.split(RegExp('/|:')).last;
    final dotIndex = fileName.lastIndexOf('.');
    if (dotIndex != -1) fileName = fileName.substring(0, dotIndex);
    return fileName.replaceAll('.', '_').upperCamelCase;
  }

  static int sortImports(String a, String b) {
    final aIsRelative = !a.contains(':');
    final bIsRelative = !b.contains(':');
    if (aIsRelative != bIsRelative) return aIsRelative ? 1 : -1;
    return a.compareTo(b);
  }
}
