// ignore_for_file: implementation_imports
import 'package:analyzer/dart/element/element.dart';
import 'package:analyzer/src/dart/element/element.dart';
import 'package:flutterx_reflection_codegen/src/encoder/target_sdk.dart';

class LibraryEncoder {
  static final Set<LibraryElement> _allLibraries = {};
  static final Set<String> usedAliases = {};
  static final Set<String> usedVariables = {};

  const LibraryEncoder._();

  static void initialize(Iterable<LibraryElement> libraries) => _allLibraries.addAll(libraries);

  static String libraryName(Element element) {
    if (element is TypeParameterElement || element is DynamicElementImpl || element is NeverElementImpl) return '';
    var libraryName = element.library?.identifier ?? '';
    if (libraryName.startsWith('asset:')) libraryName = libraryName.split('/').last;
    return libraryName;
  }

  static String libraryVariableOf(Element element, {bool use = true}) =>
      libraryVariable(libraryName(element), use: use);

  static String libraryVariable(String libraryName, {bool use = true}) {
    if (use) usedVariables.add(libraryName);
    return sdk.flutter_never.identifier(libraryName);
  }

  static String libraryAliasOf(Element element, {bool use = true}) => libraryAlias(libraryName(element), use: use);

  static String libraryAlias(String libraryName, {bool use = true}) {
    if (libraryName.isEmpty || libraryName == 'dart:core') return '';
    if (use) usedAliases.add(libraryName);
    return sdk.flutter_type_alias.identifier(libraryName);
  }

  static Set<String> exportsFor(Element element) {
    final target = LibraryEncoder.libraryName(element);
    if (target.isEmpty) return const {};
    final name = element.name;
    final result = <String>{};
    for (final library in _allLibraries) {
      final isValidExport = library.exports.any((export) {
        final exportedLibrary = export.exportedLibrary;
        if (exportedLibrary == null || LibraryEncoder.libraryName(exportedLibrary) != target) return false;
        for (final combinator in export.combinators) {
          if (combinator is HideElementCombinator) {
            if (combinator.hiddenNames.contains(name)) return true;
          } else if (combinator is ShowElementCombinator) {
            if (!combinator.shownNames.contains(name)) return false;
          }
        }
        return true;
      });
      if (isValidExport) {
        result.add(LibraryEncoder.libraryVariableOf(library));
        result.addAll(exportsFor(library));
      }
    }
    return result;
  }

  static String encodeImport(String libraryName) {
    final alias = libraryAlias(libraryName, use: false);
    return 'import \'$libraryName\'${alias.isEmpty ? '' : ' as $alias'};';
  }

  static String encodeVariable(String libraryName) =>
      'const ${LibraryEncoder.libraryVariable(libraryName, use: false)} = \'$libraryName\';';

  static bool isInternal(String libraryName) =>
      libraryName.startsWith('dart:_') || libraryName == 'dart:nativewrappers';
}
