part of '../source_encoder.dart';

class ClassTypeEncoder extends TypeEncoder<InterfaceType> {
  static const ClassTypeEncoder instance = ClassTypeEncoder._();

  @override
  ClassElementEncoder get elementEncoder => ClassElementEncoder.instance;

  const ClassTypeEncoder._() : super._(sdk.flutter_class);

  @override
  void load(Map<String, Element> elements, InterfaceType type) {
    final loadType = _typeLoader(elements);
    elementEncoder.load(elements, type.element);
    type.typeArguments.forEach(loadType);
  }

  @override
  String identifier(InterfaceType type) => elementEncoder.identifier(type.element) + nullabilityIdentifier(type);

  @override
  String reference(InterfaceType type) {
    final out = StringBuffer();
    out.write(elementEncoder.reference(type.element));
    final typeArgs = type.typeArguments;
    if (typeArgs.isNotEmpty)
      out.write('<${typeArgs.map((typeArg) => TypeEncoder.of(typeArg).reference(typeArg)).join(', ')}>');
    out.write(nullabilitySuffix(type));
    return out.toString();
  }

  @override
  String referenceDeclaration(InterfaceType type) {
    final out = StringBuffer();
    out.write(elementEncoder.reference(type.element));
    final typeArgs = type.typeArguments;
    if (typeArgs.isNotEmpty)
      out.write('<${typeArgs.map((typeArg) => TypeEncoder.of(typeArg).referenceDeclaration(typeArg)).join(', ')}>');
    out.write(nullabilitySuffix(type));
    return out.toString();
  }

  @override
  String variable(InterfaceType type) {
    final typeArgs = type.typeArguments;
    final variable = elementEncoder.variable(type.element);
    if (typeArgs.isNotEmpty)
      return '$kParametrizedFlutterClass($variable, [${typeArgs.map((typeArg) => TypeEncoder.of(typeArg).variable(typeArg)).join(', ')}])';
    return variable;
  }
}
