part of '../source_encoder.dart';

class TypeAliasTypeEncoder extends TypeEncoder<DartType> {
  static const TypeAliasTypeEncoder instance = TypeAliasTypeEncoder._();

  @override
  TypeAliasElementEncoder get elementEncoder => TypeAliasElementEncoder.instance;

  const TypeAliasTypeEncoder._() : super._(sdk.flutter_type_alias);

  @override
  void load(Map<String, Element> elements, DartType type) => elementEncoder.load(elements, type.aliasElement!);

  @override
  String identifier(DartType type) => elementEncoder.identifier(type.aliasElement!) + nullabilityIdentifier(type);

  @override
  String reference(DartType type) {
    final out = StringBuffer();
    out.write(elementEncoder.reference(type.aliasElement!));
    final typeArgs = type.aliasArguments ?? const <DartType>[];
    if (typeArgs.isNotEmpty)
      out.write('<${typeArgs.map((typeArg) => TypeEncoder.of(typeArg).reference(typeArg)).join(', ')}>');
    out.write(nullabilitySuffix(type));
    return out.toString();
  }

  @override
  String referenceDeclaration(DartType type) {
    final out = StringBuffer();
    out.write(elementEncoder.reference(type.aliasElement!));
    final typeArgs = type.aliasArguments ?? const <DartType>[];
    if (typeArgs.isNotEmpty)
      out.write('<${typeArgs.map((typeArg) => TypeEncoder.of(typeArg).referenceDeclaration(typeArg)).join(', ')}>');
    out.write(nullabilitySuffix(type));
    return out.toString();
  }

  @override
  String variable(DartType type) => elementEncoder.variable(type.aliasElement!);
}
