part of '../flutterx_reflection.dart';

/// Field of a class
///
/// [type] of this field
/// [name] of this field
/// [getter] getter implementation having type as return type and receiver as argument
/// [setter] setter implementation having as parameter the receiver and the new value.
///          If not set it means this field is final
/// [isFinal] if [setter] is null
/// [toString] describes this field accurately through [_describeFlutterField] function
/// [get] simply invoke the [getter] and return field value
/// [set] simply invoke the [setter] and set new field value
/// [field] short alias for (type as FlutterClass).field(name). If you call this you are sure that
///         type is a [FlutterClass]
/// [resolveType] returns this field retyped with actual type args or this field if not resolved
@immutable
class FlutterField {
  final FlutterType type;
  final String name;
  final Function getter;
  final Function? setter;

  bool get isFinal => setter == null;

  const FlutterField(this.type, this.name, this.getter, [this.setter]);

  @override
  String toString() => _describeFlutterField(this);

  T get<T>(dynamic receiver) => getter(receiver);

  void set(dynamic receiver, dynamic value) =>
      requireNotNull(setter, message: () => 'cannot set a final field')(receiver, value);

  FlutterField field(String name) => (type as FlutterClass).field(name);

  FlutterField resolveType(Map<FlutterTypeParameter, FlutterType> actualTypeArgs) {
    if (type is! FlutterTypeParameter) return this;
    final resolved = actualTypeArgs[type];
    return resolved == null ? this : FlutterField(resolved, name, getter, setter);
  }
}
