part of '../flutterx_reflection.dart';

/// Enumeration type (e.g. 'enum CrossAxisAlignment { start, ... }')
///
/// [token] @see [FlutterType.token]
/// [library] @see [DefinedType.library]
/// [exports] @see [DefinedType.exports]
/// [name] @see [FlutterType.name]
/// [referenceId] @see [DefinedType.referenceId]
/// [values] enum entries of this enum (e.g. CrossAxisAlignment.values)
/// [toString] @see [FlutterType.toString]
/// [isAssignableFrom] only if it's the same enum. @see [FlutterType.isAssignableFrom]
/// [entry] simply return enum entry instance by name
@immutable
class FlutterEnum implements FlutterType, DefinedType {
  @override
  final TypeToken token;

  @override
  final String library;

  @override
  final Set<String> exports;

  @override
  final String name;

  @override
  String get referenceId => DefinedType.generateReferenceId(this);

  final List<Object> values;

  const FlutterEnum({
    this.token = const TypeToken<dynamic>(),
    required this.library,
    this.exports = const {},
    required this.name,
    this.values = const [],
  });

  @override
  String toString() => _describeFlutterType(this);

  @override
  bool isAssignableFrom(FlutterType other) {
    if (other is FlutterTypeAlias) other = other.aliasedType; // handle type alias
    if (other is! FlutterEnum) return false; // other has no type args (e.g. List<String> = List)
    return other.library == library && other.name == name; // same type
  }

  dynamic entry(String name) => values.require((item) => item.toString() == name,
      message: () => 'enum entry \'$name\' not found in $this[${values.join(', ')}]');
}
