part of '../flutterx_reflection.dart';

/// Flutter [Never] type
///
/// [token] @see [FlutterType.token]
/// [library] not applicable to Never @see [DefinedType.library]
/// [exports] not applicable to Never @see [DefinedType.exports]
/// [name] @see [FlutterType.name]
/// [referenceId] @see [DefinedType.referenceId]
/// [toString] @see [FlutterType.toString]
/// [isAssignableFrom] always false: Never is not assignable. @see [FlutterType.isAssignableFrom]
@immutable
class FlutterNever implements FlutterType, DefinedType {
  @override
  TypeToken get token => const TypeToken<Never>();

  @override
  String get library => 'dart:core';

  @override
  Set<String> get exports => const {};

  @override
  String get name => 'Never';

  @override
  String get referenceId => DefinedType.generateReferenceId(this);

  const FlutterNever();

  @override
  String toString() => _describeFlutterType(this);

  @override
  bool isAssignableFrom(FlutterType other) => false;
}
