part of '../flutterx_reflection.dart';

/// Flutter type parameter (e.g. <T extends Iterable>)
///
/// [token] should never be used: since it's a generic type can only be dynamic or the same as it's bound @see [FlutterType.token]
/// [name] @see [FlutterType.name]
/// [toString] @see [FlutterType.toString]
/// [isAssignableFrom] if bound is provided return true if isAssignableFrom bound, else is always assignable. @see [FlutterType.isAssignableFrom]
@immutable
class FlutterTypeParameter implements FlutterType {
  @override
  TypeToken get token => bound?.token ?? const TypeToken<dynamic>();

  @override
  final String name;

  FlutterType? get bound => _bound?.call();

  final ValueGetter<FlutterType>? _bound;

  const FlutterTypeParameter({
    required this.name,
    ValueGetter<FlutterType>? bound,
  }) : _bound = bound;

  @override
  String toString() => _describeFlutterType(this);

  @override
  bool isAssignableFrom(FlutterType other) {
    if (other is FlutterTypeParameter) return FlutterType.equals(bound, other.bound);
    return bound?.isAssignableFrom(other) ?? true;
  }
}
