part of '../flutterx_reflection.dart';

/// Flutter [void] type
///
/// [token] @see [FlutterType.token]
/// [library] not applicable to void @see [DefinedType.library]
/// [exports] not applicable to void @see [DefinedType.exports]
/// [name] @see [FlutterType.name]
/// [referenceId] @see [DefinedType.referenceId]
/// [toString] @see [FlutterType.toString]
/// [isAssignableFrom] void is assignable only by itself. @see [FlutterType.isAssignableFrom]
@immutable
class FlutterVoid implements FlutterType, DefinedType {
  @override
  TypeToken get token => const TypeToken<void>();

  @override
  String get library => '';

  @override
  Set<String> get exports => const {};

  @override
  String get name => 'void';

  @override
  String get referenceId => DefinedType.generateReferenceId(this);

  const FlutterVoid();

  @override
  String toString() => _describeFlutterType(this);

  @override
  bool isAssignableFrom(FlutterType other) {
    if (other is FlutterTypeAlias) other = other.aliasedType; // handle type alias
    return other is FlutterVoid;
  }
}
