part of '../../flutterx_reflection.dart';

String _describeFlutterParameter(FlutterParameter parameter) {
  final out = StringBuffer();
  if (parameter.isNamed && !parameter.isOptional) out.write('required ');
  out.write(_describeFlutterType(parameter.type, false));
  if (parameter.isNullable) out.write('?');
  if (parameter.name.isNotEmpty) out.write(' ${parameter.name}');
  if (parameter.defaultValue != null) out.write(' = ${parameter.defaultValue}');
  return out.toString();
}

String _describeFlutterType(FlutterType type, [bool declaration = true]) {
  if (type is FlutterTypeAlias) return _describeFlutterTypeAlias(type, declaration);
  if (type is FlutterClass) return _describeFlutterClass(type, declaration);
  if (type is FlutterEnum) return _describeFlutterEnum(type, declaration);
  if (type is FlutterFunction) return _describeFlutterFunction(type, declaration);
  if (type is FlutterTypeParameter) return _describeFlutterTypeParameter(type, declaration);
  return type.name;
}

String _describeFlutterTypeAlias(FlutterTypeAlias type, bool declaration) {
  final out = StringBuffer();
  if (declaration) out.write('typedef ');
  out.write(type.name);
  out.write(_describeTypeArgs(type, declaration));
  if (declaration) out.write(' = ${_describeFlutterType(type.aliasedType, false)}');
  return out.toString();
}

String _describeFlutterClass(FlutterClass type, bool declaration) {
  final out = StringBuffer();
  if (declaration) {
    if (type.isAbstract) out.write('abstract ');
    out.write('class ');
  }
  out.write(type.name);
  out.write(_describeTypeArgs(type, declaration));
  return out.toString();
}

String _describeFlutterConstructor(FlutterConstructor constructor) {
  final out = StringBuffer();
  if (constructor.isConst) out.write('const ');
  out.write(constructor.declaring.name);
  if (constructor.name.isNotEmpty) out.write('.${constructor.name}');
  out.write('(${_describeParameters(constructor.parameters)})');
  return out.toString();
}

String _describeFlutterField(FlutterField field) {
  final out = StringBuffer();
  if (field.isFinal) out.write('final ');
  out.write('${field.type.name} ${field.name}');
  return out.toString();
}

String _describeFlutterEnum(FlutterEnum type, bool declaration) {
  final out = StringBuffer();
  if (declaration) out.write('enum ');
  out.write(type.name);
  if (declaration) out.write(' { ${type.values.map(describeEnum).join(', ')} }');
  return out.toString();
}

String _describeFlutterFunction(FlutterFunction type, bool declaration) {
  final out = StringBuffer();
  out.write(_describeFlutterType(type.returnType, false));
  out.write(' Function');
  out.write(_describeTypeArgs(type, true));
  out.write('(${_describeParameters(type.parameters)})');
  return out.toString();
}

String _describeFlutterTypeParameter(FlutterTypeParameter type, bool declaration) {
  final out = StringBuffer();
  out.write(type.name);
  if (declaration) type.bound?.let((bound) => out.write(' extends ${_describeFlutterType(bound, false)}'));
  return out.toString();
}

String _describeTypeArgs(RawType type, bool declaration) {
  var typeArgs = type is ParametrizedType ? type.typeArgs : type.typeParameters as Iterable<FlutterType>;
  if (type is FlutterFunction) typeArgs = type.typeFormals;
  if (typeArgs.isEmpty) return '';
  return '<${typeArgs.map((typeArg) => _describeFlutterType(typeArg, declaration && typeArg is FlutterTypeParameter)).join(', ')}>';
}

String _describeParameters(Set<FlutterParameter> parameters) {
  final out = <String>[];
  var end = '';
  for (final parameter in parameters) {
    var part = '';
    if (end.isEmpty) {
      if (parameter.isNamed) {
        part += '{';
        end = '}';
      } else if (parameter.isOptional) {
        part += ']';
        end = ']';
      }
    }
    part += _describeFlutterParameter(parameter);
    out.add(part);
  }
  return out.join(', ') + end;
}
