import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:flutterx_reflection/flutterx_reflection.dart';

import 'framework/flutter_sdk.framework.dart';

void main() {
  test('Test Fields', () {
    final data = ThemeData(primaryColor: Colors.red);
    final dataClass = FlutterSdk.lookUp('package:flutter/material.dart:ThemeData') as FlutterClass;
    assert(dataClass.token.isInstance(data));
    assert(!dataClass.token.isInstance(true));
    final primaryColor = dataClass.field('primaryColor').get(data);
    assert(primaryColor == data.primaryColor);
    assert(identical(primaryColor, data.primaryColor));
    final textTheme = dataClass.field('textTheme');
    assert(textTheme.field('bodyText1').get(textTheme.get(data)) == data.textTheme.bodyText1);
  });
  test('Test Column instantiation', () {
    final start1 = DateTime.now();
    final sColumn = FlutterSdk.lookUp('package:flutter/material.dart:Column') as FlutterClass;
    final sWidget = FlutterSdk.lookUp('package:flutter/material.dart:Widget') as FlutterClass;
    final sList = FlutterSdk.lookUp('dart:core:List') as FlutterClass;
    assert(sWidget.isAssignableFrom(sColumn));
    assert(!sColumn.isAssignableFrom(sWidget));
    assert(sList.isAssignableFrom(sList.parametrize([sWidget])));
    assert(sList.parametrize([sWidget]).isAssignableFrom(sList.parametrize([sColumn])));
    assert(!sList.parametrize([sColumn]).isAssignableFrom(sList.parametrize([sWidget])));
    final end1 = DateTime.now();
    final start2 = DateTime.now();
    final constructor = sColumn.constructor();
    final instance = sColumn.newInstance(constructor: constructor, provider: _ColumnInstanceProvider(constructor));
    final end2 = DateTime.now();
    print('Reflection: Created instance $instance');
    print('    Time for finding class: ${end1.difference(start1).inMilliseconds}ms');
    print('    Time for instantiating class: ${end2.difference(start2).inMilliseconds}ms');
  });
  testWidgets('Test Default values instantiation', (tester) async {
    for (final type in FlutterSdk.allTypes.whereType<FlutterClass>()) {
      for (final constructor in type.constructors) {
        for (final parameter in constructor.parameters) {
          if (parameter.defaultHasResolver) {
            print('Resolving default getter value of ${type.name}.${parameter.name}');
            print('   ${parameter.defaultValue}');
          }
        }
      }
    }
  });
}

class _ColumnInstanceProvider extends InstanceProvider {
  final FlutterConstructor constructor;

  _ColumnInstanceProvider(this.constructor);

  @override
  dynamic provide(ParameterContext context) {
    final result = _getValue(context);
    print('Requested ${context.actualType} ${context.parameter.name} -> return $result');
    return result;
  }

  dynamic _getValue<X>(ParameterContext context) {
    switch (context.parameter.name) {
      case 'children':
        return const <Widget>[Text('HELLO')];
      default:
        return context.parameter.defaultValue;
    }
  }
}
